from flask_wtf import FlaskForm
from wtforms import (
    BooleanField,
    DateField,
    IntegerField,
    SelectField,
    StringField,
    TextAreaField,
    validators,
)
from wtforms.validators import DataRequired, ValidationError


class ProjectForm(FlaskForm):
    title = StringField("Title", validators=[DataRequired()])
    description = TextAreaField("Description", validators=[DataRequired()])
    background_skills = TextAreaField("Background skills")
    is_student_proposal = BooleanField("Student Proposal", default=False)
    on_behalf = StringField("from", default=None)
    category = SelectField("Category", coerce=str)
    meeting_modes = [("", "N/A"), ("remote", "Remote"), ("in-person", "In person")]
    meeting_mode = SelectField(
        "Meeting type",
        choices=meeting_modes,
        coerce=str,
        validators=[
            validators.Optional(),
            validators.AnyOf([c[0] for c in meeting_modes]),
        ],
    )
    proposed_start_date = DateField(
        "Proposed start date", validators=[validators.Optional()]
    )
    duration_in_weeks = IntegerField(
        "Duration (in weeks)",
        validators=[validators.Optional(), validators.NumberRange(min=0)],
    )
    time_commitments = [
        ("", "N/A"),
        ("full-time", "Full-time"),
        ("part-time", "Part-time"),
    ]
    time_commitment = SelectField(
        "Time Commitment",
        choices=time_commitments,
        coerce=str,
        validators=[
            validators.Optional(),
            validators.AnyOf([c[0] for c in time_commitments]),
        ],
    )
    lab_usages = [
        ("", "N/A"),
        ("lab-based", "Lab-based project"),
        ("non-lab-based", "Not lab-based"),
    ]
    lab_usage = SelectField(
        "Lab usage",
        choices=lab_usages,
        coerce=str,
        validators=[
            validators.Optional(),
            validators.AnyOf([c[0] for c in lab_usages]),
        ],
    )

    def validate_on_behalf(self, field):
        if self.is_student_proposal.data and not field.data:
            raise ValidationError(
                "Student proposer cannot be empty if project is marked as student proposal."
            )

    def validate_category(self, field):
        if field == self.category and field.data == "":
            self.category.errors += "Please choose a category for the project."
            raise ValidationError("Please choose a category for the project.")
